package com.rjconsultores.ventaboletos.relatorios.impl;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.vo.impressaofiscal.ItemRelatorioVoucher;
import com.rjconsultores.ventaboletos.vo.impressaofiscal.TotalRelatorioVoucher;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

public class RelatorioAnaliticoVoucher extends Relatorio {

	private static Logger log = Logger.getLogger(RelatorioAnaliticoVoucher.class);

	List<ItemRelatorioVoucher> listdata = null;

	public RelatorioAnaliticoVoucher(final Map<String, Object> parametros, Connection conexao) throws Exception {
		super(parametros, conexao);

		this.setCustomDataSource(new DataSource(this) {
			@Override
			public void initDados() throws Exception {

				try {
					Date inicio = (Date) parametros.get("inicio");
					Date fim = (Date) parametros.get("fim");
					Empresa empresa = (Empresa) parametros.get("empresa");

					listdata = buscarMovimentosAnaliticos(inicio, fim, empresa.getEmpresaId());

				} catch (SQLException e) {
					log.error("", e);
				}
			}

		});

		this.setCollectionDataSource(new JRBeanCollectionDataSource(listdata));
	}

	public List<ItemRelatorioVoucher> buscarMovimentosAnaliticos(Date inicio, Date fim, Integer empresaId) throws SQLException {

		inicio = DateUtil.inicioFecha(inicio);
		fim = DateUtil.fimFecha(fim);

		StringBuilder sql = new StringBuilder();
		sql.append("select ");
		sql.append("	  TO_CHAR(b.fechorventa, 'dd-MM-yyyy') as dataVendaVoucher, ");
		sql.append("	  TO_CHAR(impres.fechorventa, 'dd-MM-yyyy') as dataEmissao, ");
		sql.append("	  e.nombempresa as empresa, ");
		sql.append("	  fr4.coo as numDocFiscal, ");
		sql.append("	  coalesce(b.numfoliosistema, impres.numfoliosistema) as bilhete, ");
		sql.append("	  b.boleto_id as boletoVoucher, ");
		sql.append("	  impres.boleto_id as boletoImpresso, ");
		sql.append("	  tv.desctipoventa as tipoVenda, ");
		sql.append("	  case when b.motivocancelacion_id in (31, 32) then b.preciopagado * -1 else b.preciopagado end as tarifa, ");
		sql.append("	  case when b.motivocancelacion_id in (31, 32) then b.importepedagio * -1 else b.importepedagio end as pedagio, ");
		sql.append("	  case when b.motivocancelacion_id in (31, 32) then b.importetaxaembarque * -1 else b.importetaxaembarque end as embarque, ");
		sql.append("	  case when mc.descmotivo is null then 'VOUCHER' else mc.descmotivo end as status ");
		sql.append("	from boleto b ");
		sql.append("	join empresa e on b.marca_id = e.empresa_id ");
		sql.append("	left join boleto impres on b.boleto_id = impres.boletooriginal_id ");
		sql.append("	left join fiscal_r4 fr4 on impres.boleto_id = fr4.boleto_id ");
		sql.append("	left join tipo_venta tv on tv.tipoventa_id =  coalesce(b.tipoventa_id, impres.tipoventa_id) ");
		sql.append("	left join motivo_cancelacion mc on mc.motivocancelacion_id = coalesce(b.motivocancelacion_id, impres.motivocancelacion_id) ");
		sql.append("	where b.tipoventa_id in (5,12,18) ");
		sql.append("	and b.fechorventa between ? and ? ");
		sql.append("	and b.marca_id = ? ");
		sql.append("	order by dataVendaVoucher, dataEmissao desc ");

		PreparedStatement stmt = getConexao().prepareStatement(sql.toString());
		stmt.setTimestamp(1, new java.sql.Timestamp(inicio.getTime()));
		stmt.setTimestamp(2, new java.sql.Timestamp(fim.getTime()));
		stmt.setInt(3, empresaId);

		ResultSet rset = stmt.executeQuery();
		List<ItemRelatorioVoucher> list = new ArrayList<ItemRelatorioVoucher>();
		Map<String, TotalRelatorioVoucher> totais = new HashMap<String, TotalRelatorioVoucher>();
		while (rset.next()) {
			ItemRelatorioVoucher item = new ItemRelatorioVoucher();

			item.setDataVendaVoucher(rset.getString("dataVendaVoucher"));
			item.setDataEmissao(rset.getString("dataEmissao"));
			item.setEmpresa(rset.getString("empresa"));
			item.setNumDocFiscal(rset.getString("numDocFiscal"));
			item.setBilhete(rset.getString("bilhete"));
			item.setBoletoVoucher(rset.getLong("boletoVoucher"));
			item.setBoletoImpresso(rset.getLong("boletoImpresso"));
			item.setTipoVenda(rset.getString("tipoVenda"));
			item.setTarifa(rset.getBigDecimal("tarifa"));
			item.setPedagio(rset.getBigDecimal("pedagio"));
			item.setEmbarque(rset.getBigDecimal("embarque"));
			item.setStatus(rset.getString("status"));
			
			TotalRelatorioVoucher total = null;
			if(totais.containsKey(item.getStatus())) {
				total = totais.get(item.getStatus());
			} else {
				total = new TotalRelatorioVoucher();
			}
			
			total.setStatus(item.getStatus());
			total.setTarifa(total.getTarifa().add(item.getTarifa().intValue() < 0 ? item.getTarifa().multiply(new BigDecimal(-1)) : item.getTarifa()));
			total.setPedagio(total.getPedagio().add(item.getPedagio().intValue() < 0 ? item.getPedagio().multiply(new BigDecimal(-1)) : item.getPedagio()));
			total.setEmbarque(total.getEmbarque().add(item.getEmbarque().intValue() < 0 ? item.getEmbarque().multiply(new BigDecimal(-1)) : item.getEmbarque()));
			totais.put(item.getStatus(), total);

			list.add(item);
		}
		
		parametros.put("totais", totais.values());

		if (!getConexao().isClosed())
			getConexao().close();

		return list;
	}

	@Override
	protected void processaParametros() throws Exception {
	}

}
